@@ -1,9 +1,7 @@
-#!/usr/bin/env python3
-
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
 This file is part of BORIS.
 
@@ -22,17 +20,16 @@ This file is part of BORIS.
 
 """
 
-
 import logging
 
 from PyQt5.QtCore import *
 from PyQt5.QtGui import *
 from PyQt5.QtWidgets import *
 
-from boris import dialog
-from boris.add_modifier_ui import Ui_Dialog
-from boris.config import *
-from boris.utilities import sorted_keys
+from . import dialog
+from .add_modifier_ui import Ui_Dialog
+from .config import *
+from .utilities import sorted_keys
 
 
 class addModifierDialog(QDialog, Ui_Dialog):
@@ -88,9 +85,11 @@ class addModifierDialog(QDialog, Ui_Dialog):
             self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{int(idx) + 1}")
 
         if self.tabWidgetModifiersSets.currentIndex() == -1:
-            for w in [self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers, self.pbMoveUp,
-                      self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft, self.pbMoveSetRight,
-                      self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers]:
+            for w in [
+                    self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers,
+                    self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft,
+                    self.pbMoveSetRight, self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers
+            ]:
                 w.setVisible(False)
             for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                 w.setEnabled(False)
@@ -98,8 +97,6 @@ class addModifierDialog(QDialog, Ui_Dialog):
         # set first tab as active
         self.tabMem = 0
 
-
-
     def pb_pushed(self, button):
 
         if self.leModifier.text():
@@ -112,7 +109,6 @@ class addModifierDialog(QDialog, Ui_Dialog):
         if button == "cancel":
             self.reject()
 
-
     def add_subjects(self):
         """
         add subjects as modifiers
@@ -124,9 +120,9 @@ class addModifierDialog(QDialog, Ui_Dialog):
             else:
                 self.lwModifiers.addItem(f"{subject} ({key})" if key else subject)
 
-        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                               for x in range(self.lwModifiers.count())]
-
+        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+            self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+        ]
 
     def add_modifiers_from_file(self):
         """
@@ -150,20 +146,22 @@ class addModifierDialog(QDialog, Ui_Dialog):
                                     break
                             else:
 
-                                if line.strip() not in [self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())]:
+                                if line.strip() not in [
+                                        self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+                                ]:
 
                                     if self.itemPositionMem != -1:
                                         self.lwModifiers.insertItem(self.itemPositionMem, line.strip())
                                     else:
                                         self.lwModifiers.addItem(line.strip())
 
-                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                                       for x in range(self.lwModifiers.count())]
+                self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+                    self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+                ]
             except Exception:
                 QMessageBox.warning(self, programName, f"Error reading modifiers from file:<br>{file_name}")
                 logging.warning(f"Error reading modifiers from file<br>{file_name}")
 
-
     def sort_modifiers(self):
         """
         sort modifiers
@@ -177,9 +175,9 @@ class addModifierDialog(QDialog, Ui_Dialog):
             else:
                 self.lwModifiers.addItem(modifier)
 
-        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                               for x in range(self.lwModifiers.count())]
-
+        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+            self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+        ]
 
     def set_name_changed(self):
         """
@@ -187,8 +185,8 @@ class addModifierDialog(QDialog, Ui_Dialog):
         """
         if not self.modifiers_sets_dict:
             self.modifiers_sets_dict["0"] = {"name": "", "type": SINGLE_SELECTION, "values": []}
-        self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["name"] = self.leSetName.text().strip()
-
+        self.modifiers_sets_dict[str(
+            self.tabWidgetModifiersSets.currentIndex())]["name"] = self.leSetName.text().strip()
 
     def type_changed(self):
         """
@@ -198,48 +196,45 @@ class addModifierDialog(QDialog, Ui_Dialog):
             self.modifiers_sets_dict["0"] = {"name": "", "type": SINGLE_SELECTION, "values": []}
         self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["type"] = self.cbType.currentIndex()
         # disable if modifier numeric or value from external data file
-        for obj in [self.lbValues, self.lwModifiers, self.leModifier, self.leCode, self.lbModifier, self.lbCode, self.lbCodeHelp,
-                    self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pb_add_subjects,
-                    self.pbAddModifier, self.pbModifyModifier, self.pb_load_file, self.pb_sort_modifiers]:
+        for obj in [
+                self.lbValues, self.lwModifiers, self.leModifier, self.leCode, self.lbModifier, self.lbCode,
+                self.lbCodeHelp, self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pb_add_subjects,
+                self.pbAddModifier, self.pbModifyModifier, self.pb_load_file, self.pb_sort_modifiers
+        ]:
             obj.setEnabled(self.cbType.currentIndex() not in [NUMERIC_MODIFIER, EXTERNAL_DATA_MODIFIER])
         if self.cbType.currentIndex() == EXTERNAL_DATA_MODIFIER:
             self.lbSetName.setText("Variable name")
         else:
             self.lbSetName.setText("Set name")
 
-
     def moveSetLeft(self):
         """
         move selected modifiers set left
         """
         if self.tabWidgetModifiersSets.currentIndex():
-            self.modifiers_sets_dict[
-                str(self.tabWidgetModifiersSets.currentIndex() - 1)
-            ], self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())] = (
-                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
-                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() - 1)]),
-            )
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() - 1)], self.modifiers_sets_dict[str(
+                self.tabWidgetModifiersSets.currentIndex())] = (
+                    dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
+                    dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() - 1)]),
+                )
             self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.currentIndex() - 1)
             self.tabMem = self.tabWidgetModifiersSets.currentIndex()
 
-
     def moveSetRight(self):
         """
         move selected modifiers set right
         """
         if self.tabWidgetModifiersSets.currentIndex() < self.tabWidgetModifiersSets.count() - 1:
 
-            self.modifiers_sets_dict[
-                str(self.tabWidgetModifiersSets.currentIndex() + 1)
-            ], self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())] = (
-                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
-                dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() + 1)]),
-            )
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() + 1)], self.modifiers_sets_dict[str(
+                self.tabWidgetModifiersSets.currentIndex())] = (
+                    dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]),
+                    dict(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex() + 1)]),
+                )
 
             self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.currentIndex() + 1)
             self.tabMem = self.tabWidgetModifiersSets.currentIndex()
 
-
     def moveModifierUp(self):
         """
         move up the selected modifier
@@ -249,9 +244,9 @@ class addModifierDialog(QDialog, Ui_Dialog):
             currentItem = self.lwModifiers.takeItem(currentRow)
             self.lwModifiers.insertItem(currentRow - 1, currentItem)
             self.lwModifiers.setCurrentItem(currentItem)
-            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                                   for x in range(self.lwModifiers.count())]
-
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+            ]
 
     def moveModifierDown(self):
         """
@@ -262,9 +257,9 @@ class addModifierDialog(QDialog, Ui_Dialog):
             currentItem = self.lwModifiers.takeItem(currentRow)
             self.lwModifiers.insertItem(currentRow + 1, currentItem)
             self.lwModifiers.setCurrentItem(currentItem)
-            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                                   for x in range(self.lwModifiers.count())]
-
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+            ]
 
     def addSet(self):
         """
@@ -273,30 +268,40 @@ class addModifierDialog(QDialog, Ui_Dialog):
 
         # no modifiers set
         if self.tabWidgetModifiersSets.currentIndex() == -1:
-            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {"name": "", "type": SINGLE_SELECTION, "values": []}
+            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {
+                "name": "",
+                "type": SINGLE_SELECTION,
+                "values": []
+            }
             self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{len(self.modifiers_sets_dict)}")
             self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.count() - 1)
             self.tabMem = self.tabWidgetModifiersSets.currentIndex()
 
             # set visible and available buttons and others elements
-            for w in [self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers, self.pbMoveUp,
-                      self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft, self.pbMoveSetRight,
-                      self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers]:
+            for w in [
+                    self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers,
+                    self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft,
+                    self.pbMoveSetRight, self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers
+            ]:
                 w.setVisible(True)
             for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                 w.setEnabled(True)
             return
 
         if len(self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]):
-            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {"name": "", "type": SINGLE_SELECTION, "values": []}
+            self.modifiers_sets_dict[str(len(self.modifiers_sets_dict))] = {
+                "name": "",
+                "type": SINGLE_SELECTION,
+                "values": []
+            }
             self.tabWidgetModifiersSets.addTab(QWidget(), f"Set #{len(self.modifiers_sets_dict)}")
             self.tabWidgetModifiersSets.setCurrentIndex(self.tabWidgetModifiersSets.count() - 1)
             self.tabMem = self.tabWidgetModifiersSets.currentIndex()
 
         else:
-            QMessageBox.information(self, programName,
-                                    "It is not possible to add a modifiers' set while the current modifiers' set is empty.")
-
+            QMessageBox.information(
+                self, programName,
+                "It is not possible to add a modifiers' set while the current modifiers' set is empty.")
 
     def removeSet(self):
         """
@@ -322,30 +327,30 @@ class addModifierDialog(QDialog, Ui_Dialog):
 
                 # set not visible and not available buttons and others elements
                 if self.tabWidgetModifiersSets.currentIndex() == -1:
-                    for w in [self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers, self.pbMoveUp,
-                              self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft, self.pbMoveSetRight]:
+                    for w in [
+                            self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers,
+                            self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft,
+                            self.pbMoveSetRight
+                    ]:
                         w.setVisible(False)
                     for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                         w.setEnabled(False)
 
                 if not len(self.modifiers_sets_dict):
                     # set invisible and unavailable buttons and others elements
-                    for w in [self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers, self.pbMoveUp,
-                              self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft, self.pbMoveSetRight,
-                              self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers]:
+                    for w in [
+                            self.lbSetName, self.lbType, self.lbValues, self.leSetName, self.cbType, self.lwModifiers,
+                            self.pbMoveUp, self.pbMoveDown, self.pbRemoveModifier, self.pbRemoveSet, self.pbMoveSetLeft,
+                            self.pbMoveSetRight, self.pb_add_subjects, self.pb_load_file, self.pb_sort_modifiers
+                    ]:
                         w.setVisible(False)
                     for w in [self.leModifier, self.leCode, self.pbAddModifier, self.pbModifyModifier]:
                         w.setEnabled(False)
                     return
 
-
-
-
-
         else:
             QMessageBox.information(self, programName, "It is not possible to remove the last modifiers' set.")
 
-
     def modifyModifier(self):
         """
         modify modifier <- arrow
@@ -360,25 +365,23 @@ class addModifierDialog(QDialog, Ui_Dialog):
             self.leModifier.setText(txt.split("(")[0].strip())
             self.leCode.setText(code)
 
-            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())][
-                "values"
-            ].remove(self.lwModifiers.currentItem().text())
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"].remove(
+                self.lwModifiers.currentItem().text())
 
             self.itemPositionMem = self.lwModifiers.currentRow()
             self.lwModifiers.takeItem(self.lwModifiers.currentRow())
         else:
             QMessageBox.information(self, programName, "Select a modifier to modify from the modifiers set")
 
-
     def removeModifier(self):
         """
         remove modifier from set
         """
         if self.lwModifiers.currentIndex().row() >= 0:
             self.lwModifiers.takeItem(self.lwModifiers.currentIndex().row())
-            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                                   for x in range(self.lwModifiers.count())]
-
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+            ]
 
     def addModifier(self):
         """
@@ -388,18 +391,16 @@ class addModifierDialog(QDialog, Ui_Dialog):
         txt = self.leModifier.text().strip()
         for c in CHAR_FORBIDDEN_IN_MODIFIERS:
             if c in txt:
-                QMessageBox.critical(self, programName,
-                                     (f"The character <b>{c}</b> is not allowed.<br>"
-                                      "The following characters are not allowed in modifiers:<br>"
-                                      f"<b>{CHAR_FORBIDDEN_IN_MODIFIERS}</b>"))
+                QMessageBox.critical(self, programName, (f"The character <b>{c}</b> is not allowed.<br>"
+                                                         "The following characters are not allowed in modifiers:<br>"
+                                                         f"<b>{CHAR_FORBIDDEN_IN_MODIFIERS}</b>"))
                 self.leModifier.setFocus()
                 return
 
         if txt:
 
             if txt in [self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())]:
-                QMessageBox.critical(self, programName,
-                                     f"The modifier <b>{txt}</b> is already in the list")
+                QMessageBox.critical(self, programName, f"The modifier <b>{txt}</b> is already in the list")
                 return
 
             if not self.modifiers_sets_dict:
@@ -407,8 +408,10 @@ class addModifierDialog(QDialog, Ui_Dialog):
 
             if len(self.leCode.text().strip()) > 1:
                 if self.leCode.text().strip().upper() not in function_keys.values():
-                    QMessageBox.critical(self, programName,
-                                         "The modifier key code can not exceed one key\nSelect one key or a function key (F1, F2 ... F12)")
+                    QMessageBox.critical(
+                        self, programName,
+                        "The modifier key code can not exceed one key\nSelect one key or a function key (F1, F2 ... F12)"
+                    )
                     self.leCode.setFocus()
                     return
 
@@ -422,13 +425,14 @@ class addModifierDialog(QDialog, Ui_Dialog):
 
                 # check if code already exists
                 if not self.modifiers_sets_dict:
-                    self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())] = {"name": "",
-                                                                                                 "type": SINGLE_SELECTION,
-                                                                                                 "values": []}
+                    self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())] = {
+                        "name": "",
+                        "type": SINGLE_SELECTION,
+                        "values": []
+                    }
 
-                if "(" + self.leCode.text().strip() + ")" in " ".join(
-                    self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"]
-                ):
+                if "(" + self.leCode.text().strip() + ")" in " ".join(self.modifiers_sets_dict[str(
+                        self.tabWidgetModifiersSets.currentIndex())]["values"]):
 
                     QMessageBox.critical(self, programName,
                                          f"The shortcut code <b>{self.leCode.text().strip()}</b> already exists!")
@@ -441,8 +445,9 @@ class addModifierDialog(QDialog, Ui_Dialog):
             else:
                 self.lwModifiers.addItem(txt)
 
-            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [self.lwModifiers.item(x).text()
-                                                                                                   for x in range(self.lwModifiers.count())]
+            self.modifiers_sets_dict[str(self.tabWidgetModifiersSets.currentIndex())]["values"] = [
+                self.lwModifiers.item(x).text() for x in range(self.lwModifiers.count())
+            ]
             self.leModifier.setText("")
             self.leCode.setText("")
 
@@ -450,7 +455,6 @@ class addModifierDialog(QDialog, Ui_Dialog):
             QMessageBox.critical(self, programName, "No modifier to add!")
             self.leModifier.setFocus()
 
-
     def tabWidgetModifiersSets_changed(self, tabIndex):
         """
         user changed the tab widget
@@ -476,14 +480,14 @@ class addModifierDialog(QDialog, Ui_Dialog):
                 self.cbType.setCurrentIndex(self.modifiers_sets_dict[str(tabIndex)]["type"])
                 self.lwModifiers.addItems(self.modifiers_sets_dict[str(tabIndex)]["values"])
 
-
     def getModifiers(self):
         """
         returns modifiers as string
         """
         keys_to_delete = []
         for idx in self.modifiers_sets_dict:
-            if self.modifiers_sets_dict[idx]["type"] in [SINGLE_SELECTION, MULTI_SELECTION] and not self.modifiers_sets_dict[idx]["values"]:
+            if self.modifiers_sets_dict[idx]["type"] in [SINGLE_SELECTION, MULTI_SELECTION
+                                                        ] and not self.modifiers_sets_dict[idx]["values"]:
                 keys_to_delete.append(idx)
 
         for idx in keys_to_delete:
