@@ -1,9 +1,7 @@
-#!/usr/bin/env python3
-
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
 This file is part of BORIS.
 
@@ -37,7 +35,6 @@ NA = "NA"
 VLC_TIMER_OUT = 200
 SPECTRO_TIMER = 200
 
-
 function_keys = {
     16777264: "F1",
     16777265: "F2",
@@ -79,6 +76,7 @@ COMPACT_TIME_BUDGET_FORMAT = "compact"
 LONG_TIME_BUDGET_FORMAT = "long"
 DEFAULT_TIME_BUDGET_FORMAT = LONG_TIME_BUDGET_FORMAT
 
+TIME_BIN_SIZE = "time bin size"
 
 CODING_MAP_RESIZE_W = 640
 CODING_MAP_RESIZE_H = 640
@@ -143,7 +141,6 @@ BEHAVIOR_CODE = "code"
 BEHAVIOR_KEY = "key"
 BEHAVIOR_CATEGORY = "category"
 
-
 # fields for event configuration
 fields = {
     "type": 0,
@@ -226,11 +223,9 @@ tw_obs_fields, pj_obs_fields = {}, {}
 for idx, field in enumerate(tw_events_fields):
     tw_obs_fields[field] = idx
 
-
 for idx, field in enumerate(pj_events_fields):
     pj_obs_fields[field] = idx
 
-
 EVENT_TIME_FIELD_IDX = 0
 
 SUBJECT_EVENT_FIELD = 1  # to be removed after check
@@ -254,6 +249,8 @@ HHMMSSZZZ = "hh:mm:ss.zzz"
 S = "s"
 
 START_FROM_CURRENT_TIME = "start_from_current_time"
+START_FROM_CURRENT_EPOCH_TIME = "start_from_current_epoch_time"
+
 SCAN_SAMPLING_TIME = "scan_sampling_time"
 
 NEW = "new"
@@ -288,14 +285,12 @@ SET_OF_VALUES_idx = 2
 TIMESTAMP = "timestamp"
 TIMESTAMP_idx = 3
 
-
 TIME_FULL_OBS = "full obs"
 TIME_EVENTS = "limit to events"
 TIME_ARBITRARY_INTERVAL = "time interval"
 
 AVAILABLE_INDEP_VAR_TYPES = [NUMERIC, TEXT, SET_OF_VALUES, TIMESTAMP]
 
-
 INDEPENDENT_VARIABLES = "independent_variables"
 OBSERVATIONS = "observations"
 
@@ -369,20 +364,38 @@ subtitlesColors = [
 ]
 
 CATEGORY_COLORS_LIST = [
-    "#FF96CC",
-    "#96FF9C",
-    "#CCFFFE",
-    "#EEFF70",
-    "#FF4F64",
-    "#F8BF15",
-    "#3DC7AD",
+    "magenta",
+    "yellow",
+    "lime",
+    "darksalmon",
+    "purple",
+    "orange",
+    "maroon",
+    "silver",
+    "slateblue",
+    "hotpink",
+    "steelblue",
+    "darkgoldenrod",
+    "aqua",
+    "aquamarine",
+    "beige",
+    "bisque",
+    "black",
+    "blanchedalmond",
+    "blueviolet",
+    "brown",
+    "burlywood",
+    "cadetblue",
 ]
 
+CODING_PAD_CONFIG = "coding pad configuration"
+CODING_PAD_GEOMETRY = "coding pad geometry"
+# NO_COLOR_CODING_PAD = "#777777"
+
 SPECTROGRAM_COLOR_MAPS = ["viridis", "inferno", "plasma", "magma", "gray", "YlOrRd"]
 SPECTROGRAM_DEFAULT_COLOR_MAP = "viridis"
 SPECTROGRAM_DEFAULT_TIME_INTERVAL = 10
 
-
 # see matplotlib.colors.cnames.keys()
 # https://xkcd.com/color/rgb/
 
@@ -393,6 +406,7 @@ SPECTROGRAM_DEFAULT_TIME_INTERVAL = 10
 ACTIVE_MEASUREMENTS_COLOR = "lime"
 PASSIVE_MEASUREMENTS_COLOR = "red"
 
+# see matplotlib for color name
 BEHAVIORS_PLOT_COLORS = [
     "tab:blue",
     "tab:orange",
@@ -561,10 +575,10 @@ EMPTY_PROJECT = {
     CONVERTERS: {},
 }
 
-
-INIT_PARAM = {DISPLAY_SUBTITLES: False,
-              SAVE_FRAMES: DISK,
-              MEMORY_FOR_FRAMES: DEFAULT_MEMORY_FOR_FRAMES,
-              ADAPT_FAST_JUMP: ADAPT_FAST_JUMP_DEFAULT,
-              TIME_BUDGET_FORMAT: DEFAULT_TIME_BUDGET_FORMAT,
-              }
+INIT_PARAM = {
+    DISPLAY_SUBTITLES: False,
+    SAVE_FRAMES: DISK,
+    MEMORY_FOR_FRAMES: DEFAULT_MEMORY_FOR_FRAMES,
+    ADAPT_FAST_JUMP: ADAPT_FAST_JUMP_DEFAULT,
+    TIME_BUDGET_FORMAT: DEFAULT_TIME_BUDGET_FORMAT,
+}
