@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
 This file is part of BORIS.
 
@@ -21,10 +21,12 @@ This file is part of BORIS.
 """
 
 import logging
-import re
 import sys
+import platform
+import traceback
+import datetime as dt
+import pathlib as pl
 
-import tablib
 from PyQt5.QtCore import *
 from PyQt5.QtGui import *
 from PyQt5.QtWidgets import *
@@ -37,6 +39,56 @@ from boris import version
 from boris.config import *
 
 
+def global_error_message(exception_type, exception_value, traceback_object):
+    """
+    global error management
+    save error using loggin.critical and append error message to ~/boris_error.log
+    """
+
+    error_text: str = (f"BORIS version: {version.__version__}\n"
+                       f"OS: {platform.uname().system} {platform.uname().release} {platform.uname().version}\n"
+                       f"CPU: {platform.uname().machine} {platform.uname().processor}\n"
+                       f"Python {platform.python_version()} ({'64-bit' if sys.maxsize > 2**32 else '32-bit'})\n"
+                       f"Qt {QT_VERSION_STR} - PyQt {PYQT_VERSION_STR}\n"
+                       f"{dt.datetime.now():%Y-%m-%d %H:%M}\n\n")
+    error_text += "".join(traceback.format_exception(exception_type, exception_value, traceback_object))
+
+    logging.critical(error_text)
+
+    # append to boris_error.log file
+    with open(pl.Path("~").expanduser() / "boris_error.log", "a") as f_out:
+        f_out.write(error_text + "\n")
+        f_out.write("-" * 80 + "\n")
+
+    # copy to clipboard
+    cb = QApplication.clipboard()
+    cb.clear(mode=cb.Clipboard)
+    cb.setText(error_text, mode=cb.Clipboard)
+
+    error_text: str = error_text.replace("\r\n", "\n").replace("\n", "<br>")
+
+    text: str = (f"<b>An error has occured</b>:<br><br>"
+                 f"{error_text}<br>"
+                 "to improve the software please report this problem at:<br>"
+                 '<a href="https://github.com/olivierfriard/BORIS/issues">'
+                 "https://github.com/olivierfriard/BORIS/issues</a><br>"
+                 "Please no screenshot, the error message was copied to the clipboard.<br><br>"
+                 "Thank you for your collaboration!")
+
+    errorbox = QMessageBox()
+    errorbox.setWindowTitle("BORIS error occured")
+    errorbox.setText(text)
+    errorbox.setTextFormat(Qt.RichText)
+    errorbox.setStandardButtons(QMessageBox.Abort)
+
+    _ = errorbox.addButton("Ignore and try to continue", QMessageBox.RejectRole)
+
+    ret = errorbox.exec_()
+
+    if ret == QMessageBox.Abort:
+        sys.exit(1)
+
+
 def MessageDialog(title, text, buttons):
     response = ""
     message = QMessageBox()
@@ -51,23 +103,22 @@ def MessageDialog(title, text, buttons):
     return message.clickedButton().text()
 
 
-def error_message_box(task, error_type, error_file_name, error_lineno):
+def error_message_box(task, error_type, error_file_name,
+                      error_lineno):  # do NOT use this function directly, use error_message function
     """
     show a critical dialog
-    do not use this function directly, use error_message
+
     """
-    QMessageBox.critical(None, programName,
-                         (f"BORIS version: {version.__version__}<br>"
-                          f"An error occured during the execution of: <b>{task}</b>.<br>"
-                          f"Error: {error_type}<br>"
-                          f"in {error_file_name} "
-                          f"at line # {error_lineno}<br><br>"
-                          "to improve the software please report this problem at:<br>"
-                          '<a href="https://github.com/olivierfriard/BORIS/issues">'
-                          'https://github.com/olivierfriard/BORIS/issues</a><br>'
-                          "or by email (See the About page on the BORIS web site.<br><br>"
-                          "Thank you for your collaboration!"
-                          ))
+    QMessageBox.critical(None, programName, (f"BORIS version: {version.__version__}<br>"
+                                             f"An error occured during the execution of: <b>{task}</b>.<br>"
+                                             f"Error: {error_type}<br>"
+                                             f"in {error_file_name} "
+                                             f"at line # {error_lineno}<br><br>"
+                                             "to improve the software please report this problem at:<br>"
+                                             '<a href="https://github.com/olivierfriard/BORIS/issues">'
+                                             'https://github.com/olivierfriard/BORIS/issues</a><br>'
+                                             "or by email (See the About page on the BORIS web site.<br><br>"
+                                             "Thank you for your collaboration!"))
 
 
 def error_message(task: str, exc_info: tuple) -> None:
@@ -77,7 +128,6 @@ def error_message(task: str, exc_info: tuple) -> None:
     """
     error_type, error_file_name, error_lineno = utilities.error_info(exc_info)
     logging.critical(f"Error during {task}: {error_type} in {error_file_name} at line #{error_lineno}")
-    #logging.critical(f"Error in function '{sys._getframe().f_code.co_name}'': {error_type} {error_file_name} {error_lineno}")
     error_message_box(task, error_type, error_file_name, error_lineno)
 
 
@@ -216,7 +266,6 @@ class Input_dialog(QDialog):
         self.setWindowTitle("title")
 
 
-
 class DuplicateBehaviorCode(QDialog):
     """
     let user show between behaviors that are coded by same key
@@ -363,7 +412,6 @@ class Ask_time(QDialog):
         self.setWindowTitle("Time")
 
 
-
 class EditSelectedEvents(QDialog):
     """
     "edit selected events" dialog box
@@ -413,7 +461,6 @@ class EditSelectedEvents(QDialog):
 
         self.setLayout(hbox)
 
-
     def rb_changed(self):
 
         self.newText.setEnabled(not self.rbComment.isChecked())
@@ -430,7 +477,6 @@ class EditSelectedEvents(QDialog):
         if self.rbComment.isChecked():
             self.newText.clear()
 
-
     def pbOK_clicked(self):
         if not self.rbSubject.isChecked() and not self.rbBehavior.isChecked() and not self.rbComment.isChecked():
             QMessageBox.warning(None, programName, "You must select a field to be edited",
@@ -531,7 +577,6 @@ class FindReplaceEvents(QWidget):
         self.cbSubject.setChecked(False)
         hbox.addWidget(self.cbSubject)
 
-
         self.cbBehavior = QCheckBox("Behavior")
         self.cbBehavior.setChecked(False)
         hbox.addWidget(self.cbBehavior)
@@ -640,6 +685,7 @@ class Results_dialog(QDialog):
     """
     widget for visualizing text output
     """
+
     def __init__(self):
         super().__init__()
 
@@ -675,7 +721,6 @@ class Results_dialog(QDialog):
 
         self.resize(540, 640)
 
-
     def save_results(self):
         """
         save content of self.ptText
@@ -696,6 +741,7 @@ class ResultsWidget(QWidget):
     """
     widget for visualizing text output
     """
+
     def __init__(self):
         super().__init__()
 
@@ -724,7 +770,6 @@ class ResultsWidget(QWidget):
 
         self.resize(540, 640)
 
-
     def save_results(self):
         """
         save content of self.ptText
@@ -816,7 +861,6 @@ class View_explore_project_results(QWidget):
 
         # self.resize(540, 640)
 
-
     def tw_cellDoubleClicked(self, r, c):
 
         self.double_click_signal.emit(self.tw.item(r, 0).text(), int(self.tw.item(r, 1).text()))
@@ -854,7 +898,6 @@ def choose_obs_subj_behav_category(pj: dict,
                                    by_category: bool = False,
                                    show_time: bool = False,
                                    timeFormat: str = HHMMSS):
-
     """
     show window for:
       - selection of subjects
@@ -891,7 +934,6 @@ def choose_obs_subj_behav_category(pj: dict,
 
     paramPanelWindow.frm_time_interval.setEnabled(False)
 
-
     if timeFormat == HHMMSS:
         paramPanelWindow.start_time.set_format_hhmmss()
         paramPanelWindow.end_time.set_format_hhmmss()
@@ -914,7 +956,6 @@ def choose_obs_subj_behav_category(pj: dict,
     paramPanelWindow.start_time.set_time(0)
     paramPanelWindow.end_time.set_time(maxTime)
 
-
     # hide max time
     if not maxTime:
         paramPanelWindow.frm_time.setVisible(False)
@@ -951,7 +992,8 @@ def choose_obs_subj_behav_category(pj: dict,
     logging.debug(f'selectedSubjects: {selectedSubjects}')
 
     if selected_observations:
-        observedBehaviors = paramPanelWindow.extract_observed_behaviors(selected_observations, selectedSubjects)  # not sorted
+        observedBehaviors = paramPanelWindow.extract_observed_behaviors(selected_observations,
+                                                                        selectedSubjects)  # not sorted
     else:
         # load all behaviors
         observedBehaviors = [pj[ETHOGRAM][x][BEHAVIOR_CODE] for x in pj[ETHOGRAM]]
@@ -962,7 +1004,11 @@ def choose_obs_subj_behav_category(pj: dict,
         categories = pj[BEHAVIORAL_CATEGORIES][:]
         # check if behavior not included in a category
         try:
-            if "" in [pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY] for idx in pj[ETHOGRAM] if BEHAVIOR_CATEGORY in pj[ETHOGRAM][idx]]:
+            if "" in [
+                    pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY]
+                    for idx in pj[ETHOGRAM]
+                    if BEHAVIOR_CATEGORY in pj[ETHOGRAM][idx]
+            ]:
                 categories += [""]
         except Exception:
             categories = ["###no category###"]
@@ -990,9 +1036,11 @@ def choose_obs_subj_behav_category(pj: dict,
 
         for behavior in [pj[ETHOGRAM][x][BEHAVIOR_CODE] for x in utilities.sorted_keys(pj[ETHOGRAM])]:
 
-            if ((categories == ["###no category###"])
-                or (behavior in [pj[ETHOGRAM][x][BEHAVIOR_CODE] for x in pj[ETHOGRAM]
-                    if BEHAVIOR_CATEGORY in pj[ETHOGRAM][x] and pj[ETHOGRAM][x][BEHAVIOR_CATEGORY] == category])):
+            if ((categories == ["###no category###"]) or (behavior in [
+                    pj[ETHOGRAM][x][BEHAVIOR_CODE]
+                    for x in pj[ETHOGRAM]
+                    if BEHAVIOR_CATEGORY in pj[ETHOGRAM][x] and pj[ETHOGRAM][x][BEHAVIOR_CATEGORY] == category
+            ])):
 
                 paramPanelWindow.item = QListWidgetItem(behavior)
                 if behavior in observedBehaviors:
@@ -1010,8 +1058,7 @@ def choose_obs_subj_behav_category(pj: dict,
                 paramPanelWindow.lwBehaviors.addItem(paramPanelWindow.item)
 
     if not paramPanelWindow.exec_():
-        return {SELECTED_SUBJECTS: [],
-                SELECTED_BEHAVIORS: []}
+        return {SELECTED_SUBJECTS: [], SELECTED_BEHAVIORS: []}
 
     selectedSubjects = paramPanelWindow.selectedSubjects
     selectedBehaviors = paramPanelWindow.selectedBehaviors
@@ -1041,11 +1088,12 @@ def choose_obs_subj_behav_category(pj: dict,
     if paramPanelWindow.rb_interval.isChecked():
         time_param = TIME_ARBITRARY_INTERVAL
 
-    return {SELECTED_SUBJECTS: selectedSubjects,
-            SELECTED_BEHAVIORS: selectedBehaviors,
-            INCLUDE_MODIFIERS: paramPanelWindow.cbIncludeModifiers.isChecked(),
-            EXCLUDE_BEHAVIORS: paramPanelWindow.cbExcludeBehaviors.isChecked(),
-            "time": time_param,
-            START_TIME: startTime,
-            END_TIME: endTime
-            }
+    return {
+        SELECTED_SUBJECTS: selectedSubjects,
+        SELECTED_BEHAVIORS: selectedBehaviors,
+        INCLUDE_MODIFIERS: paramPanelWindow.cbIncludeModifiers.isChecked(),
+        EXCLUDE_BEHAVIORS: paramPanelWindow.cbExcludeBehaviors.isChecked(),
+        "time": time_param,
+        START_TIME: startTime,
+        END_TIME: endTime
+    }
