@@ -1,9 +1,7 @@
-#!/usr/bin/env python3
-
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
 This file is part of BORIS.
 
@@ -22,14 +20,14 @@ This file is part of BORIS.
 
 """
 
-
 import logging
 import os
 
-from boris.config import BEHAVIORS_PLOT_COLORS, CANCEL
-from boris.dialog import MessageDialog
-from boris.preferences_ui import Ui_prefDialog
-from PyQt5.QtWidgets import QDialog, QFileDialog
+from .config import (BEHAVIORS_PLOT_COLORS, CATEGORY_COLORS_LIST, CANCEL)
+from .dialog import MessageDialog
+from .preferences_ui import Ui_prefDialog
+
+from PyQt5.QtWidgets import (QDialog, QFileDialog)
 
 
 class Preferences(QDialog, Ui_prefDialog):
@@ -44,7 +42,8 @@ class Preferences(QDialog, Ui_prefDialog):
         self.rb_save_frames_on_disk.toggled.connect(self.rb_frames_mem_disk)
         self.cb_reset_frames_memory.clicked.connect(self.reset_frames_memory)
 
-        self.pb_reset_colors.clicked.connect(self.reset_colors)
+        self.pb_reset_behav_colors.clicked.connect(self.reset_behav_colors)
+        self.pb_reset_category_colors.clicked.connect(self.reset_category_colors)
 
         self.pb_refresh.clicked.connect(self.refresh_preferences)
         self.pbOK.clicked.connect(self.accept)
@@ -53,7 +52,6 @@ class Preferences(QDialog, Ui_prefDialog):
         self.flag_refresh = False
         self.flag_reset_frames_memory = False
 
-
     def rb_frames_mem_disk(self):
         """
         change where extracted frames will be saved: disk or memory
@@ -64,7 +62,6 @@ class Preferences(QDialog, Ui_prefDialog):
         for w in [self.lb_storage_dir]:
             w.setEnabled(self.rb_save_frames_on_disk.isChecked())
 
-
     def reset_frames_memory(self):
         """
         reset memory used for frames storage.
@@ -72,19 +69,16 @@ class Preferences(QDialog, Ui_prefDialog):
         """
         self.flag_reset_frames_memory = self.cb_reset_frames_memory.isChecked()
 
-
     def refresh_preferences(self):
         """
         allow user to delete the config file (.boris)
         """
-        if MessageDialog("BORIS",
-                         ("Refresh will re-initialize "
-                          "all your preferences and close BORIS"),
+        if MessageDialog("BORIS", ("Refresh will re-initialize "
+                                   "all your preferences and close BORIS"),
                          [CANCEL, "Refresh preferences"]) == "Refresh preferences":
             self.flag_refresh = True
             self.accept()
 
-
     def browseFFmpegCacheDir(self):
         """
         allow user select a cache dir for ffmpeg images
@@ -96,11 +90,18 @@ class Preferences(QDialog, Ui_prefDialog):
         if FFmpegCacheDir:
             self.leFFmpegCacheDir.setText(FFmpegCacheDir)
 
-
-    def reset_colors(self):
+    def reset_behav_colors(self):
         """
         reset behavior colors to default
         """
-        self.te_plot_colors.setPlainText("\n".join(BEHAVIORS_PLOT_COLORS))
+        self.te_behav_colors.setPlainText("\n".join(BEHAVIORS_PLOT_COLORS))
 
         logging.debug("reset behaviors colors to default")
+
+    def reset_category_colors(self):
+        """
+        reset category colors to default
+        """
+        self.te_category_colors.setPlainText("\n".join(CATEGORY_COLORS_LIST))
+
+        logging.debug("reset category colors to default")
