@@ -1,7 +1,7 @@
 """
 BORIS
 Behavioral Observation Research Interactive Software
-Copyright 2012-2020 Olivier Friard
+Copyright 2012-2022 Olivier Friard
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,22 +19,106 @@ Copyright 2012-2020 Olivier Friard
   MA 02110-1301, USA.
 """
 
-
 import json
 import logging
 import os
 import pathlib
 import sys
-from decimal import *
+from decimal import Decimal as dec
 from shutil import copyfile
 import gzip
+from . import portion as I
 
-from boris import db_functions
-from boris import dialog
-from boris import select_observations
+from . import db_functions
+from . import dialog
+from . import select_observations
 import tablib
-from boris import utilities
-from boris.config import *
+from . import utilities
+from .config import *
+
+
+def check_observation_exhaustivity(events: list, ethogram: list, state_events_list: list = []):
+    """
+    check if observation is continous
+    if ethogram not empty state events list is determined else
+    """
+
+    def interval_len(interval):
+        if interval.empty:
+            return dec(0)
+        else:
+            return sum([x.upper - x.lower for x in interval])
+
+    def interval_number(interval):
+        if interval.empty:
+            return dec(0)
+        else:
+            return len(interval)
+
+    if ethogram:
+        state_events_list = [ethogram[x][BEHAVIOR_CODE] for x in ethogram if STATE in ethogram[x][TYPE].upper()]
+
+    events_interval = {}
+    mem_events_interval = {}
+
+    for event in events:
+        if event[EVENT_SUBJECT_FIELD_IDX] not in events_interval:
+            events_interval[event[EVENT_SUBJECT_FIELD_IDX]] = {}
+            mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]] = {}
+
+        if event[EVENT_BEHAVIOR_FIELD_IDX] not in events_interval[event[EVENT_SUBJECT_FIELD_IDX]]:
+            events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]] = I.empty()
+            mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]] = []
+
+        if event[EVENT_BEHAVIOR_FIELD_IDX] in state_events_list:
+            mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]].append(
+                event[EVENT_TIME_FIELD_IDX])
+            if len(mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]]) == 2:
+                events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]] |= \
+                        I.closedopen(mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]][0],
+                                    mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]][1])
+                mem_events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]] = []
+        else:
+            events_interval[event[EVENT_SUBJECT_FIELD_IDX]][event[EVENT_BEHAVIOR_FIELD_IDX]] |= I.singleton(
+                event[EVENT_TIME_FIELD_IDX])
+
+    if events:
+        obs_theo_dur = max(events)[EVENT_TIME_FIELD_IDX] - min(events)[EVENT_TIME_FIELD_IDX]
+    else:
+        obs_theo_dur = dec("0")
+
+    count = 0
+    total_duration = 0
+    for subject in events_interval:
+        '''
+        print("\n\nsubject", subject)
+        '''
+
+        tot_behav_for_subject = I.empty()
+        for behav in events_interval[subject]:
+            tot_behav_for_subject |= events_interval[subject][behav]
+        '''
+        print(f"tot\n{tot_behav_for_subject}")
+        '''
+
+        obs_real_dur = interval_len(tot_behav_for_subject)
+
+        if obs_real_dur >= obs_theo_dur:
+            obs_real_dur = obs_theo_dur
+
+        total_duration += obs_real_dur
+        '''
+        print(f"theo dur: {obs_theo_dur}  real: {obs_real_dur}  exhaustiv %: {obs_real_dur/obs_theo_dur}")
+        '''
+    '''
+    print(f"  {total_duration/(len(events_interval) * obs_theo_dur) } ")
+    '''
+    if len(events_interval) and obs_theo_dur:
+        exhausivity_percent = total_duration / (len(events_interval) * obs_theo_dur) * 100
+    else:
+        exhausivity_percent = 0
+
+    return round(exhausivity_percent, 1)
 
 
 def behavior_category(ethogram: dict) -> dict:
@@ -134,7 +218,11 @@ def check_state_events_obs(obsId: str, ethogram: dict, observation: dict, time_f
 
     for subject in sorted(set(subjects)):
 
-        behaviors = [event[EVENT_BEHAVIOR_FIELD_IDX] for event in observation[EVENTS] if event[EVENT_SUBJECT_FIELD_IDX] == subject]
+        behaviors = [
+            event[EVENT_BEHAVIOR_FIELD_IDX]
+            for event in observation[EVENTS]
+            if event[EVENT_SUBJECT_FIELD_IDX] == subject
+        ]
 
         for behavior in sorted(set(behaviors)):
             if behavior not in ethogram_behaviors:
@@ -145,9 +233,8 @@ def check_state_events_obs(obsId: str, ethogram: dict, observation: dict, time_f
                     flagStateEvent = True
                     lst, memTime = [], {}
                     for event in [
-                        event
-                        for event in observation[EVENTS]
-                        if event[EVENT_BEHAVIOR_FIELD_IDX] == behavior and event[EVENT_SUBJECT_FIELD_IDX] == subject
+                            event for event in observation[EVENTS]
+                            if event[EVENT_BEHAVIOR_FIELD_IDX] == behavior and event[EVENT_SUBJECT_FIELD_IDX] == subject
                     ]:
 
                         behav_modif = [event[EVENT_BEHAVIOR_FIELD_IDX], event[EVENT_MODIFIER_FIELD_IDX]]
@@ -160,31 +247,28 @@ def check_state_events_obs(obsId: str, ethogram: dict, observation: dict, time_f
                             memTime[str(behav_modif)] = event[EVENT_TIME_FIELD_IDX]
 
                     for event in lst:
-                        out += (
-                            'The behavior <b>{behavior}</b> {modifier} is not PAIRED for subject'
-                            ' "<b>{subject}</b>" at <b>{time}</b><br>'
-                        ).format(
-                            behavior=behavior,
-                            modifier=("(modifier " + event[1] + ") ") if event[1] else "",
-                            subject=subject if subject else NO_FOCAL_SUBJECT,
-                            time=memTime[str(event)] if time_format == S else utilities.seconds2time(memTime[str(event)]),
-                        )
+                        out += ('The behavior <b>{behavior}</b> {modifier} is not PAIRED for subject'
+                                ' "<b>{subject}</b>" at <b>{time}</b><br>').format(
+                                    behavior=behavior,
+                                    modifier=("(modifier " + event[1] + ") ") if event[1] else "",
+                                    subject=subject if subject else NO_FOCAL_SUBJECT,
+                                    time=memTime[str(event)]
+                                    if time_format == S else utilities.seconds2time(memTime[str(event)]),
+                                )
 
     return (False, out) if out else (True, "No problem detected")
 
 
-
 def check_project_integrity(pj: dict,
                             time_format: str,
                             project_file_name: str,
                             media_file_available: bool = True) -> str:
-
     """
     check project integrity
     check if behaviors in observations are in ethogram
     check unpaired state events
-    check if behavior belong to behavioral category that do not more exists
-    check for leading and trialing spaces and special chars in modifiers
+    check if behavior belong to behavioral category that do not more exist
+    check for leading and trailing spaces and special chars in modifiers
     check if media file are available
     check if media length available
 
@@ -218,11 +302,9 @@ def check_project_integrity(pj: dict,
                 if pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY]:
                     if pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY] not in pj[BEHAVIORAL_CATEGORIES]:
                         out += "<br><br>" if out else ""
-                        out += (
-                            f"The behavior <b>{pj[ETHOGRAM][idx][BEHAVIOR_CODE]}</b> belongs "
-                            f"to the behavioral category <b>{pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY]}</b> "
-                            "that is no more in behavioral categories list."
-                        )
+                        out += (f"The behavior <b>{pj[ETHOGRAM][idx][BEHAVIOR_CODE]}</b> belongs "
+                                f"to the behavioral category <b>{pj[ETHOGRAM][idx][BEHAVIOR_CATEGORY]}</b> "
+                                "that is no more in behavioral categories list.")
 
         # check for leading/trailing spaces/special chars in modifiers defined in ethogram
         for idx in pj[ETHOGRAM]:
@@ -269,10 +351,7 @@ def check_project_integrity(pj: dict,
         return str(sys.exc_info()[1])
 
 
-def create_subtitles(pj: dict,
-                     selected_observations: list,
-                     parameters: dict,
-                     export_dir: str) -> tuple:
+def create_subtitles(pj: dict, selected_observations: list, parameters: dict, export_dir: str) -> tuple:
     """
     create subtitles for selected observations, subjects and behaviors
 
@@ -302,16 +381,15 @@ def create_subtitles(pj: dict,
             return "", ""
         else:
             return (
-                """<font color="{}">""".format(
-                    subtitlesColors[parameters["selected subjects"].index(row["subject"]) % len(subtitlesColors)]
-                ),
+                """<font color="{}">""".format(subtitlesColors[parameters["selected subjects"].index(row["subject"]) %
+                                                               len(subtitlesColors)]),
                 "</font>",
             )
 
     try:
-        ok, msg, db_connector = db_functions.load_aggregated_events_in_db(
-            pj, parameters["selected subjects"], selected_observations, parameters["selected behaviors"]
-        )
+        ok, msg, db_connector = db_functions.load_aggregated_events_in_db(pj, parameters["selected subjects"],
+                                                                          selected_observations,
+                                                                          parameters["selected behaviors"])
         if not ok:
             return False, msg
 
@@ -322,13 +400,11 @@ def create_subtitles(pj: dict,
             if pj[OBSERVATIONS][obsId][TYPE] in [LIVE]:
                 out = ""
                 cursor.execute(
-                    (
-                        "SELECT subject, behavior, start, stop, type, modifiers FROM aggregated_events "
-                        "WHERE observation = ? AND subject in ({}) "
-                        "AND behavior in ({}) "
-                        "ORDER BY start"
-                    ).format(",".join(["?"] * len(parameters["selected subjects"])),
-                             ",".join(["?"] * len(parameters["selected behaviors"]))),
+                    ("SELECT subject, behavior, start, stop, type, modifiers FROM aggregated_events "
+                     "WHERE observation = ? AND subject in ({}) "
+                     "AND behavior in ({}) "
+                     "ORDER BY start").format(",".join(["?"] * len(parameters["selected subjects"])),
+                                              ",".join(["?"] * len(parameters["selected behaviors"]))),
                     [obsId] + parameters["selected subjects"] + parameters["selected behaviors"],
                 )
 
@@ -338,19 +414,21 @@ def create_subtitles(pj: dict,
                         modifiers_str = "\n{}".format(row["modifiers"].replace("|", ", "))
                     else:
                         modifiers_str = ""
-                    out += ("{idx}\n" "{start} --> {stop}\n" "{col1}{subject}: {behavior}" "{modifiers}" "{col2}\n\n").format(
-                        idx=idx + 1,
-                        start=utilities.seconds2time(row["start"]).replace(".", ","),
-                        stop=utilities.seconds2time(row["stop"] if row["type"] == STATE else row["stop"] + POINT_EVENT_ST_DURATION).replace(
-                            ".", ","
-                        ),
-                        col1=col1,
-                        col2=col2,
-                        subject=row["subject"],
-                        behavior=row["behavior"],
-                        modifiers=modifiers_str,
-                    )
-
+                    out += ("{idx}\n"
+                            "{start} --> {stop}\n"
+                            "{col1}{subject}: {behavior}"
+                            "{modifiers}"
+                            "{col2}\n\n").format(
+                                idx=idx + 1,
+                                start=utilities.seconds2time(row["start"]).replace(".", ","),
+                                stop=utilities.seconds2time(row["stop"] if row["type"] == STATE else row["stop"] +
+                                                            POINT_EVENT_ST_DURATION).replace(".", ","),
+                                col1=col1,
+                                col2=col2,
+                                subject=row["subject"],
+                                behavior=row["behavior"],
+                                modifiers=modifiers_str,
+                            )
                 '''
                 file_name = str(pathlib.Path(pathlib.Path(export_dir) / utilities.safeFileName(obsId)).with suffix(".srt"))
                 '''
@@ -376,16 +454,14 @@ def create_subtitles(pj: dict,
                         out = ""
 
                         cursor.execute(
-                            (
-                                "SELECT subject, behavior, type, start, stop, modifiers FROM aggregated_events "
-                                "WHERE observation = ? AND start BETWEEN ? and ? "
-                                "AND subject in ({}) "
-                                "AND behavior in ({}) "
-                                "ORDER BY start"
-                            ).format(
-                                ",".join(["?"] * len(parameters["selected subjects"])),
-                                ",".join(["?"] * len(parameters["selected behaviors"])),
-                            ),
+                            ("SELECT subject, behavior, type, start, stop, modifiers FROM aggregated_events "
+                             "WHERE observation = ? AND start BETWEEN ? and ? "
+                             "AND subject in ({}) "
+                             "AND behavior in ({}) "
+                             "ORDER BY start").format(
+                                 ",".join(["?"] * len(parameters["selected subjects"])),
+                                 ",".join(["?"] * len(parameters["selected behaviors"])),
+                             ),
                             [obsId, init, end] + parameters["selected subjects"] + parameters["selected behaviors"],
                         )
 
@@ -396,19 +472,22 @@ def create_subtitles(pj: dict,
                             else:
                                 modifiers_str = ""
 
-                            out += ("{idx}\n" "{start} --> {stop}\n" "{col1}{subject}: {behavior}" "{modifiers}" "{col2}\n\n").format(
-                                idx=idx + 1,
-                                start=utilities.seconds2time(row["start"] - init).replace(".", ","),
-                                stop=utilities.seconds2time(
-                                    (row["stop"] if row["type"] == STATE else row["stop"] + POINT_EVENT_ST_DURATION) - init
-                                ).replace(".", ","),
-                                col1=col1,
-                                col2=col2,
-                                subject=row["subject"],
-                                behavior=row["behavior"],
-                                modifiers=modifiers_str,
-                            )
-
+                            out += (
+                                "{idx}\n"
+                                "{start} --> {stop}\n"
+                                "{col1}{subject}: {behavior}"
+                                "{modifiers}"
+                                "{col2}\n\n").format(
+                                    idx=idx + 1,
+                                    start=utilities.seconds2time(row["start"] - init).replace(".", ","),
+                                    stop=utilities.seconds2time((row["stop"] if row["type"] == STATE else row["stop"] +
+                                                                 POINT_EVENT_ST_DURATION) - init).replace(".", ","),
+                                    col1=col1,
+                                    col2=col2,
+                                    subject=row["subject"],
+                                    behavior=row["behavior"],
+                                    modifiers=modifiers_str,
+                                )
                         '''
                         file_name = str(pathlib.Path(pathlib.Path(export_dir) / pathlib.Path(mediaFile).name).with suffix(".srt"))
                         '''
@@ -427,10 +506,7 @@ def create_subtitles(pj: dict,
         return False, str(sys.exc_info()[1])
 
 
-def export_observations_list(pj: dict,
-                             selected_observations: list,
-                             file_name: str,
-                             output_format: str) -> bool:
+def export_observations_list(pj: dict, selected_observations: list, file_name: str, output_format: str) -> bool:
     """
     create file with a list of selected observations
 
@@ -479,10 +555,10 @@ def export_observations_list(pj: dict,
                 else:
                     indep_var.append("")
 
-        data.append(
-            [obs_id, pj[OBSERVATIONS][obs_id]["date"], pj[OBSERVATIONS][obs_id]["description"], subjects, ", ".join(media_files)]
-            + indep_var
-        )
+        data.append([
+            obs_id, pj[OBSERVATIONS][obs_id]["date"], pj[OBSERVATIONS][obs_id]["description"], subjects, ", ".join(
+                media_files)
+        ] + indep_var)
 
     if output_format in ["tsv", "csv", "html"]:
         try:
@@ -523,11 +599,10 @@ def remove_media_files_path(pj):
                         pj[OBSERVATIONS][obs_id][FILE][n_player][idx] = p
                         if MEDIA_INFO in pj[OBSERVATIONS][obs_id]:
                             for info in [LENGTH, "hasAudio", "hasVideo", "fps"]:
-                                if (info in pj[OBSERVATIONS][obs_id][MEDIA_INFO]
-                                        and media_file in pj[OBSERVATIONS][obs_id][MEDIA_INFO][info]):
-                                    pj[OBSERVATIONS][obs_id][MEDIA_INFO][info][p] = pj[OBSERVATIONS][obs_id][MEDIA_INFO][info][
-                                        media_file
-                                    ]
+                                if (info in pj[OBSERVATIONS][obs_id][MEDIA_INFO] and
+                                        media_file in pj[OBSERVATIONS][obs_id][MEDIA_INFO][info]):
+                                    pj[OBSERVATIONS][obs_id][MEDIA_INFO][info][p] = pj[OBSERVATIONS][obs_id][
+                                        MEDIA_INFO][info][media_file]
                                     del pj[OBSERVATIONS][obs_id][MEDIA_INFO][info][media_file]
 
     return dict(pj)
@@ -558,6 +633,22 @@ def media_full_path(media_file: str, project_file_name: str) -> str:
             return ""
 
 
+def observed_interval(observation: dict):
+    """
+    Observed interval for observation
+
+    Args:
+        observation (dict): observation dictionary
+
+    Returns:
+        List of 2 Decimals: time of first observed event, time of last observed event
+    """
+    if observation[EVENTS]:
+        return (min(observation[EVENTS])[EVENT_TIME_FIELD_IDX], max(observation[EVENTS])[EVENT_TIME_FIELD_IDX])
+    else:
+        return (dec("0.0"), dec("0.0"))
+
+
 def observation_total_length(observation: dict):
     """
     Total length of media file(s) for observation
@@ -581,11 +672,11 @@ def observation_total_length(observation: dict):
         if observation[EVENTS]:
             obs_length = max(observation[EVENTS])[EVENT_TIME_FIELD_IDX]
         else:
-            obs_length = Decimal("0.0")
+            obs_length = dec("0.0")
         return obs_length
 
     if observation[TYPE] == MEDIA:
-        media_max_total_length = Decimal("0.0")
+        media_max_total_length = dec("0.0")
 
         media_total_length = {}
 
@@ -593,12 +684,12 @@ def observation_total_length(observation: dict):
             if not observation[FILE][nplayer]:
                 continue
 
-            media_total_length[nplayer] = Decimal("0.0")
+            media_total_length[nplayer] = dec("0.0")
             for mediaFile in observation[FILE][nplayer]:
                 mediaLength = 0
                 try:
                     mediaLength = observation[MEDIA_INFO][LENGTH][mediaFile]
-                    media_total_length[nplayer] += Decimal(mediaLength)
+                    media_total_length[nplayer] += dec(mediaLength)
                 except Exception:
                     logging.critical(f"media length not found for {mediaFile}")
                     mediaLength = -1
@@ -606,7 +697,7 @@ def observation_total_length(observation: dict):
                     break
 
         if -1 in [media_total_length[x] for x in media_total_length]:
-            return Decimal("-1")
+            return dec("-1")
 
         # totalMediaLength = max([total_media_length[x] for x in total_media_length])
 
@@ -620,7 +711,7 @@ def observation_total_length(observation: dict):
 
     logging.critical("observation not LIVE nor MEDIA")
 
-    return Decimal("0.0")
+    return dec("0.0")
 
 
 def observation_length(pj, selected_observations: list) -> tuple:
@@ -635,11 +726,11 @@ def observation_length(pj, selected_observations: list) -> tuple:
         float: maximum media length for all observations
         float: total media length for all observations
     """
-    selectedObsTotalMediaLength = Decimal("0.0")
+    selectedObsTotalMediaLength = dec("0.0")
     max_obs_length = 0
     for obs_id in selected_observations:
         obs_length = observation_total_length(pj[OBSERVATIONS][obs_id])
-        if obs_length in [Decimal("0"), Decimal("-1")]:
+        if obs_length in [dec("0"), dec("-1")]:
             selectedObsTotalMediaLength = -1
             break
         max_obs_length = max(max_obs_length, obs_length)
@@ -650,8 +741,7 @@ def observation_length(pj, selected_observations: list) -> tuple:
         # propose to user to use max event time
         if dialog.MessageDialog(programName,
                                 (f"A media length is not available for the observation <b>{obs_id}</b>.<br>"
-                                 "Use last event time as media length?"),
-                                [YES, NO]) == YES:
+                                 "Use last event time as media length?"), [YES, NO]) == YES:
             maxTime = 0  # max length for all events all subjects
             max_length = 0
             for obs_id in selected_observations:
@@ -666,12 +756,11 @@ def observation_length(pj, selected_observations: list) -> tuple:
 
         else:
             max_obs_length = -1
-            selectedObsTotalMediaLength = Decimal("-1")
+            selectedObsTotalMediaLength = dec("-1")
 
     return max_obs_length, selectedObsTotalMediaLength
 
 
-
 def events_start_stop(ethogram, events):
     """
     returns events with status (START/STOP or POINT)
@@ -697,19 +786,12 @@ def events_start_stop(ethogram, events):
         # check if code is state
         if code in state_events_list:
             # how many code before with same subject?
-            if (
-                len(
-                    [
-                        x[EVENT_BEHAVIOR_FIELD_IDX]
-                        for x in events
-                        if x[EVENT_BEHAVIOR_FIELD_IDX] == code and
-                        x[EVENT_TIME_FIELD_IDX] < time and
-                        x[EVENT_SUBJECT_FIELD_IDX] == subject and
-                        x[EVENT_MODIFIER_FIELD_IDX] == modifier
-                    ]
-                )
-                % 2
-            ):  # test if odd
+            if (len([
+                    x[EVENT_BEHAVIOR_FIELD_IDX]
+                    for x in events
+                    if x[EVENT_BEHAVIOR_FIELD_IDX] == code and x[EVENT_TIME_FIELD_IDX] < time and
+                    x[EVENT_SUBJECT_FIELD_IDX] == subject and x[EVENT_MODIFIER_FIELD_IDX] == modifier
+            ]) % 2):  # test if odd
                 flag = STOP
             else:
                 flag = START
@@ -721,8 +803,7 @@ def events_start_stop(ethogram, events):
     return events_flagged
 
 
-def extract_observed_subjects(pj: dict,
-                              selected_observations: list) -> list:
+def extract_observed_subjects(pj: dict, selected_observations: list) -> list:
     """
     extract unique subjects present in observations list
 
@@ -796,15 +877,13 @@ def open_project_json(projectFileName: str) -> tuple:
                 projectChanged = True
 
     # check if project file version is newer than current BORIS project file version
-    if "project_format_version" in pj and Decimal(pj["project_format_version"]) > Decimal(project_format_version):
+    if "project_format_version" in pj and dec(pj["project_format_version"]) > dec(project_format_version):
         return (
             projectFileName,
             projectChanged,
             {
-                "error": (
-                    "This project file was created with a more recent version of BORIS.<br>"
-                    f"You must update BORIS to <b>v. >= {pj['project_format_version']}</b> to open this project"
-                )
+                "error": ("This project file was created with a more recent version of BORIS.<br>"
+                          f"You must update BORIS to <b>v. >= {pj['project_format_version']}</b> to open this project")
             },
             msg,
         )
@@ -839,16 +918,17 @@ def open_project_json(projectFileName: str) -> tuple:
             key, name = pj[SUBJECTS][idx]
             pj[SUBJECTS][idx] = {"key": key, "name": name, "description": ""}
 
-        msg = (f"The project file was converted to the new format (v. {project_format_version}) in use with your version of BORIS.<br>"
-               "Choose a new file name for saving it."
-              )
+        msg = (
+            f"The project file was converted to the new format (v. {project_format_version}) in use with your version of BORIS.<br>"
+            "Choose a new file name for saving it.")
         projectFileName = ""
 
     # update modifiers to JSON format
 
     # check if project format version < 4 (modifiers were str)
     project_lowerthan4 = False
-    if "project_format_version" in pj and utilities.versiontuple(pj["project_format_version"]) < utilities.versiontuple("4.0"):
+    if "project_format_version" in pj and utilities.versiontuple(
+            pj["project_format_version"]) < utilities.versiontuple("4.0"):
         for idx in pj[ETHOGRAM]:
             if pj[ETHOGRAM][idx]["modifiers"]:
                 if isinstance(pj[ETHOGRAM][idx]["modifiers"], str):
@@ -856,13 +936,18 @@ def open_project_json(projectFileName: str) -> tuple:
                     modif_set_list = pj[ETHOGRAM][idx]["modifiers"].split("|")
                     modif_set_dict = {}
                     for modif_set in modif_set_list:
-                        modif_set_dict[str(len(modif_set_dict))] = {"name": "", "type": SINGLE_SELECTION, "values": modif_set.split(",")}
+                        modif_set_dict[str(len(modif_set_dict))] = {
+                            "name": "",
+                            "type": SINGLE_SELECTION,
+                            "values": modif_set.split(",")
+                        }
                     pj[ETHOGRAM][idx]["modifiers"] = dict(modif_set_dict)
             else:
                 pj[ETHOGRAM][idx]["modifiers"] = {}
 
         if not project_lowerthan4:
-            msg = "The project version was updated from {} to {}".format(pj["project_format_version"], project_format_version)
+            msg = "The project version was updated from {} to {}".format(pj["project_format_version"],
+                                                                         project_format_version)
             pj["project_format_version"] = project_format_version
             projectChanged = True
 
@@ -882,11 +967,11 @@ def open_project_json(projectFileName: str) -> tuple:
 
                 for media_file_path in pj[OBSERVATIONS][obs]["file"][player]:
                     # FIX: ffmpeg path
-                    ret, msg = utilities.check_ffmpeg_path()
+                    ret, ffmpeg_path = utilities.check_ffmpeg_path()
                     if not ret:
                         return projectFileName, projectChanged, {"error": "FFmpeg path not found"}, ""
                     else:
-                        ffmpeg_bin = msg
+                        ffmpeg_bin = ffmpeg_path
 
                     r = utilities.accurate_media_analysis(ffmpeg_bin, media_file_path)
 
@@ -897,24 +982,26 @@ def open_project_json(projectFileName: str) -> tuple:
                         pj[OBSERVATIONS][obs][MEDIA_INFO]["hasAudio"][media_file_path] = r["has_audio"]
                         project_updated, projectChanged = True, True
                     else:  # file path not found
-                        if (
-                            "media_file_info" in pj[OBSERVATIONS][obs] and
-                            len(pj[OBSERVATIONS][obs]["media_file_info"]) == 1 and
-                            len(pj[OBSERVATIONS][obs][FILE][PLAYER1]) == 1 and
-                            len(pj[OBSERVATIONS][obs][FILE][PLAYER2]) == 0
-                        ):
+                        if ("media_file_info" in pj[OBSERVATIONS][obs] and
+                                len(pj[OBSERVATIONS][obs]["media_file_info"]) == 1 and
+                                len(pj[OBSERVATIONS][obs][FILE][PLAYER1]) == 1 and
+                                len(pj[OBSERVATIONS][obs][FILE][PLAYER2]) == 0):
                             media_md5_key = list(pj[OBSERVATIONS][obs]["media_file_info"].keys())[0]
                             # duration
                             pj[OBSERVATIONS][obs][MEDIA_INFO] = {
-                                LENGTH: {media_file_path: pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["video_length"] / 1000}
+                                LENGTH: {
+                                    media_file_path:
+                                        pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["video_length"] / 1000
+                                }
                             }
                             projectChanged = True
 
                             # FPS
                             if "nframe" in pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]:
                                 pj[OBSERVATIONS][obs][MEDIA_INFO][FPS] = {
-                                    media_file_path: pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["nframe"]
-                                                     / (pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["video_length"] / 1000)
+                                    media_file_path:
+                                        pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["nframe"] /
+                                        (pj[OBSERVATIONS][obs]["media_file_info"][media_md5_key]["video_length"] / 1000)
                                 }
                             else:
                                 pj[OBSERVATIONS][obs][MEDIA_INFO][FPS] = {media_file_path: 0}
@@ -930,7 +1017,8 @@ def open_project_json(projectFileName: str) -> tuple:
             for player in pj[OBSERVATIONS][obs][FILE]:
                 pj[OBSERVATIONS][obs][MEDIA_INFO]["offset"][player] = 0.0
             if pj[OBSERVATIONS][obs]["time offset second player"]:
-                pj[OBSERVATIONS][obs][MEDIA_INFO]["offset"]["2"] = float(pj[OBSERVATIONS][obs]["time offset second player"])
+                pj[OBSERVATIONS][obs][MEDIA_INFO]["offset"]["2"] = float(
+                    pj[OBSERVATIONS][obs]["time offset second player"])
 
             del pj[OBSERVATIONS][obs]["time offset second player"]
             project_lowerthan7 = True
@@ -938,17 +1026,13 @@ def open_project_json(projectFileName: str) -> tuple:
             msg = (
                 f"The project file was converted to the new format (v. {project_format_version}) in use with your version of BORIS.<br>"
                 f"Please note that this new version will NOT be compatible with previous BORIS versions "
-                f"(&lt; v. {project_format_version}).<br>"
-            )
+                f"(&lt; v. {project_format_version}).<br>")
 
             projectChanged = True
 
-
     if project_lowerthan7:
 
-        msg = (
-            f"The project was updated to the current project version ({project_format_version})."
-        )
+        msg = (f"The project was updated to the current project version ({project_format_version}).")
 
         try:
             old_project_file_name = projectFileName.replace(".boris", f".v{pj['project_format_version']}.boris")
@@ -980,7 +1064,6 @@ def event_type(code: str, ethogram: dict) -> str:
     return None
 
 
-
 def fix_unpaired_state_events(obsId, ethogram, observation, fix_at_time):
     """
     fix unpaired state events in observation
@@ -1003,7 +1086,11 @@ def fix_unpaired_state_events(obsId, ethogram, observation, fix_at_time):
 
     for subject in sorted(set(subjects)):
 
-        behaviors = [event[EVENT_BEHAVIOR_FIELD_IDX] for event in observation[EVENTS] if event[EVENT_SUBJECT_FIELD_IDX] == subject]
+        behaviors = [
+            event[EVENT_BEHAVIOR_FIELD_IDX]
+            for event in observation[EVENTS]
+            if event[EVENT_SUBJECT_FIELD_IDX] == subject
+        ]
 
         for behavior in sorted(set(behaviors)):
             if (behavior in ethogram_behaviors) and (STATE in event_type(behavior, ethogram).upper()):
@@ -1011,9 +1098,8 @@ def fix_unpaired_state_events(obsId, ethogram, observation, fix_at_time):
                 flagStateEvent = True
                 lst, memTime = [], {}
                 for event in [
-                    event
-                    for event in observation[EVENTS]
-                    if event[EVENT_BEHAVIOR_FIELD_IDX] == behavior and event[EVENT_SUBJECT_FIELD_IDX] == subject
+                        event for event in observation[EVENTS]
+                        if event[EVENT_BEHAVIOR_FIELD_IDX] == behavior and event[EVENT_SUBJECT_FIELD_IDX] == subject
                 ]:
 
                     behav_modif = [event[EVENT_BEHAVIOR_FIELD_IDX], event[EVENT_MODIFIER_FIELD_IDX]]
@@ -1029,8 +1115,8 @@ def fix_unpaired_state_events(obsId, ethogram, observation, fix_at_time):
 
                     last_event_time = max([fix_at_time] + [x[0] for x in closing_events_to_add])
 
-                    closing_events_to_add.append(
-                        [last_event_time + Decimal("0.001"), subject, behavior, event[1], ""]  # modifiers  # comment
-                    )
+                    closing_events_to_add.append([last_event_time + dec("0.001"), subject, behavior, event[1],
+                                                  ""]  # modifiers  # comment
+                                                )
 
     return closing_events_to_add
