@@ -1,3 +1,25 @@
+"""
+BORIS
+Behavioral Observation Research Interactive Software
+Copyright 2012-2022 Olivier Friard
+
+This file is part of BORIS.
+
+  BORIS is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 3 of the License, or
+  any later version.
+
+  BORIS is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not see <http://www.gnu.org/licenses/>.
+
+"""
+
 import sys
 import logging
 from boris import dialog
@@ -6,9 +28,7 @@ from boris.config import *
 import boris.utilities as utilities
 from boris import param_panel
 
-from PyQt5.QtWidgets import (QFileDialog, QTableWidgetItem,
-                             QApplication, QMessageBox,
-                             QListWidgetItem)
+from PyQt5.QtWidgets import (QFileDialog, QTableWidgetItem, QApplication, QMessageBox, QListWidgetItem)
 from PyQt5.QtCore import (Qt)
 from PyQt5.QtGui import (QColor, QFont)
 
@@ -32,14 +52,14 @@ def check_text_file_type(rows):
 def import_from_text_file(self):
 
     if self.twBehaviors.rowCount():
-        response = dialog.MessageDialog(programName,
-                                        "There are behaviors already configured. Do you want to append behaviors or replace them?",
-                                        ['Append', 'Replace', CANCEL])
+        response = dialog.MessageDialog(
+            programName, "There are behaviors already configured. Do you want to append behaviors or replace them?",
+            ['Append', 'Replace', CANCEL])
         if response == CANCEL:
             return
 
     fn = QFileDialog().getOpenFileName(self, "Import behaviors from text file", "",
-                                            "Text files (*.txt *.tsv *.csv);;All files (*)")
+                                       "Text files (*.txt *.tsv *.csv);;All files (*)")
     fileName = fn[0] if type(fn) is tuple else fn
 
     if fileName:
@@ -56,10 +76,9 @@ def import_from_text_file(self):
                 try:
                     rows.append(row.decode("utf-8"))
                 except Exception:
-                    QMessageBox.critical(None, programName,
-                                            (f"Error while reading file\nThe line # {idx}\n"
-                                            f"{row}\ncontains characters that are not readable."),
-                                            QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+                    QMessageBox.critical(None, programName, (f"Error while reading file\nThe line # {idx}\n"
+                                                             f"{row}\ncontains characters that are not readable."),
+                                         QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
                     return
                 idx += 1
 
@@ -68,8 +87,9 @@ def import_from_text_file(self):
             logging.debug(f"fields separator: {fieldSeparator}  fields number: {fieldsNumber}")
 
             if fieldSeparator is None:
-                QMessageBox.critical(self, programName,
-                                        "Separator character not found! Use plain text file and TAB or comma as value separator")
+                QMessageBox.critical(
+                    self, programName,
+                    "Separator character not found! Use plain text file and TAB or comma as value separator")
             else:
 
                 for row in rows:
@@ -85,8 +105,15 @@ def import_from_text_file(self):
                     if fieldsNumber > 4:
                         type_, key, code, description = row.split(fieldSeparator)[:4]
 
-                    behavior = {"key": key, "code": code, "description": description, "modifiers": "",
-                                "excluded": "", "coding map": "", "category": ""}
+                    behavior = {
+                        "key": key,
+                        "code": code,
+                        "description": description,
+                        "modifiers": "",
+                        "excluded": "",
+                        "coding map": "",
+                        "category": ""
+                    }
 
                     self.twBehaviors.setRowCount(self.twBehaviors.rowCount() + 1)
 
@@ -107,9 +134,7 @@ def import_from_text_file(self):
 
                         self.twBehaviors.setItem(self.twBehaviors.rowCount() - 1, behavioursFields[field_type], item)
         except Exception:
-            error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-            logging.critical(f"Error in function '{sys._getframe().f_code.co_name}': {error_type} {error_file_name} {error_lineno}")
-            dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
+            dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_behaviors_from_clipboard(self):
@@ -121,15 +146,14 @@ def import_behaviors_from_clipboard(self):
         cb = QApplication.clipboard()
         cb_text = cb.text()
         if not cb_text:
-            QMessageBox.warning(None, programName,
-                                    "The clipboard is empty",
-                                    QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+            QMessageBox.warning(None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default,
+                                QMessageBox.NoButton)
             return
 
         if self.twBehaviors.rowCount():
-            response = dialog.MessageDialog(programName,
-                                            "Some behaviors are already configured. Do you want to append behaviors or replace them?",
-                                            ["Append", "Replace", CANCEL])
+            response = dialog.MessageDialog(
+                programName, "Some behaviors are already configured. Do you want to append behaviors or replace them?",
+                ["Append", "Replace", CANCEL])
             if response == CANCEL:
                 return
 
@@ -143,8 +167,8 @@ def import_behaviors_from_clipboard(self):
         if len(set([len(x.split("\t")) for x in cb_text_splitted])) != 1:
             QMessageBox.warning(None, programName,
                                 ("The clipboard content does not have a constant number of fields.<br>"
-                                    "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
-                                    QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+                                 "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
+                                QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
             return
 
         for row in cb_text_splitted:
@@ -152,7 +176,8 @@ def import_behaviors_from_clipboard(self):
                 behavior = {"type": DEFAULT_BEHAVIOR_TYPE}
                 for idx, field in enumerate(row.split("\t")):
                     if idx == 0:
-                        behavior["type"] = STATE_EVENT if STATE in field.upper() else (POINT_EVENT if POINT in field.upper() else "")
+                        behavior["type"] = STATE_EVENT if STATE in field.upper() else (
+                            POINT_EVENT if POINT in field.upper() else "")
                     if idx == 1:
                         behavior["key"] = field.strip() if len(field.strip()) == 1 else ""
                     if idx == 2:
@@ -176,10 +201,7 @@ def import_behaviors_from_clipboard(self):
 
                     self.twBehaviors.setItem(self.twBehaviors.rowCount() - 1, behavioursFields[field_type], item)
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
-
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_from_JWatcher(self):
@@ -188,13 +210,14 @@ def import_from_JWatcher(self):
     """
     try:
         if self.twBehaviors.rowCount():
-            response = dialog.MessageDialog(programName,
-                                            "There are behaviors already configured. Do you want to append behaviors or replace them?",
-                                            ["Append", "Replace", CANCEL])
+            response = dialog.MessageDialog(
+                programName, "There are behaviors already configured. Do you want to append behaviors or replace them?",
+                ["Append", "Replace", CANCEL])
             if response == CANCEL:
                 return
 
-        fn = QFileDialog().getOpenFileName(self, "Import behaviors from JWatcher", "", "Global Definition File (*.gdf);;All files (*)")
+        fn = QFileDialog().getOpenFileName(self, "Import behaviors from JWatcher", "",
+                                           "Global Definition File (*.gdf);;All files (*)")
         fileName = fn[0] if type(fn) is tuple else fn
 
         if fileName:
@@ -215,8 +238,15 @@ def import_from_JWatcher(self):
                     if idx < len(rows) and "Behavior.description." in rows[idx + 1]:
                         description = rows[idx + 1].split("=")[-1]
 
-                    behavior = {"key": key, "code": code, "description": description,
-                                "modifiers": "", "excluded": "", "coding map": "", "category": ""}
+                    behavior = {
+                        "key": key,
+                        "code": code,
+                        "description": description,
+                        "modifiers": "",
+                        "excluded": "",
+                        "coding map": "",
+                        "category": ""
+                    }
 
                     self.twBehaviors.setRowCount(self.twBehaviors.rowCount() + 1)
 
@@ -232,9 +262,7 @@ def import_from_JWatcher(self):
 
                         self.twBehaviors.setItem(self.twBehaviors.rowCount() - 1, behavioursFields[field_type], item)
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}'': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_subjects_from_clipboard(self):
@@ -245,15 +273,14 @@ def import_subjects_from_clipboard(self):
         cb = QApplication.clipboard()
         cb_text = cb.text()
         if not cb_text:
-            QMessageBox.warning(None, programName,
-                                    "The clipboard is empty",
-                                    QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
+            QMessageBox.warning(None, programName, "The clipboard is empty", QMessageBox.Ok | QMessageBox.Default,
+                                QMessageBox.NoButton)
             return
 
         if self.twSubjects.rowCount():
-            response = dialog.MessageDialog(programName,
-                                            "Some subjects are already configured. Do you want to append subjects or replace them?",
-                                            ["Append", "Replace", CANCEL])
+            response = dialog.MessageDialog(
+                programName, "Some subjects are already configured. Do you want to append subjects or replace them?",
+                ["Append", "Replace", CANCEL])
             if response == CANCEL:
                 return
 
@@ -265,7 +292,7 @@ def import_subjects_from_clipboard(self):
         if len(set([len(x.split("\t")) for x in cb_text_splitted])) != 1:
             QMessageBox.warning(None, programName,
                                 ("The clipboard content does not have a constant number of fields.<br>"
-                                    "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
+                                 "From your spreadsheet: CTRL + A (select all cells), CTRL + C (copy to clipboard)"),
                                 QMessageBox.Ok | QMessageBox.Default, QMessageBox.NoButton)
             return
 
@@ -286,9 +313,7 @@ def import_subjects_from_clipboard(self):
                     item = QTableWidgetItem(subject.get(field_name, ""))
                     self.twSubjects.setItem(self.twSubjects.rowCount() - 1, idx, item)
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def select_behaviors(title="Record value from external data file",
@@ -312,16 +337,18 @@ def select_behaviors(title="Record value from external data file",
         paramPanelWindow.resize(800, 600)
         paramPanelWindow.setWindowTitle(title)
         paramPanelWindow.lbBehaviors.setText(text)
-        for w in [paramPanelWindow.lwSubjects, paramPanelWindow.pbSelectAllSubjects, paramPanelWindow.pbUnselectAllSubjects,
-                paramPanelWindow.pbReverseSubjectsSelection, paramPanelWindow.lbSubjects, paramPanelWindow.cbIncludeModifiers,
-                paramPanelWindow.cbExcludeBehaviors, paramPanelWindow.frm_time]:
+        for w in [
+                paramPanelWindow.lwSubjects, paramPanelWindow.pbSelectAllSubjects,
+                paramPanelWindow.pbUnselectAllSubjects, paramPanelWindow.pbReverseSubjectsSelection,
+                paramPanelWindow.lbSubjects, paramPanelWindow.cbIncludeModifiers, paramPanelWindow.cbExcludeBehaviors,
+                paramPanelWindow.frm_time
+        ]:
             w.setVisible(False)
 
         if behavioral_categories:
             categories = behavioral_categories
             # check if behavior not included in a category
-            if "" in [ethogram[idx][BEHAVIOR_CATEGORY] for idx in ethogram
-                    if BEHAVIOR_CATEGORY in ethogram[idx]]:
+            if "" in [ethogram[idx][BEHAVIOR_CATEGORY] for idx in ethogram if BEHAVIOR_CATEGORY in ethogram[idx]]:
                 categories += [""]
         else:
             categories = ["###no category###"]
@@ -348,10 +375,11 @@ def select_behaviors(title="Record value from external data file",
             # check if behavior type must be shown
             for behavior in [ethogram[x][BEHAVIOR_CODE] for x in utilities.sorted_keys(ethogram)]:
 
-                if ((categories == ["###no category###"]) or
-                (behavior in [ethogram[x][BEHAVIOR_CODE] for x in ethogram
-                                if BEHAVIOR_CATEGORY in ethogram[x] and
-                                    ethogram[x][BEHAVIOR_CATEGORY] == category])):
+                if ((categories == ["###no category###"]) or (behavior in [
+                        ethogram[x][BEHAVIOR_CODE]
+                        for x in ethogram
+                        if BEHAVIOR_CATEGORY in ethogram[x] and ethogram[x][BEHAVIOR_CATEGORY] == category
+                ])):
 
                     paramPanelWindow.item = QListWidgetItem(behavior)
                     paramPanelWindow.item.setCheckState(Qt.Unchecked)
@@ -370,18 +398,15 @@ def select_behaviors(title="Record value from external data file",
 
         return []
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}'': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_behaviors_from_project(self):
     try:
 
         fn = QFileDialog().getOpenFileName(self, "Import behaviors from project file", "",
-                                            ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)")
-                                            )
+                                           ("Project files (*.boris *.boris.gz);;"
+                                            "All files (*)"))
         file_name = fn[0] if type(fn) is tuple else fn
 
         if file_name:
@@ -394,9 +419,8 @@ def import_behaviors_from_project(self):
             # configuration of behaviours
             if ETHOGRAM in project and project[ETHOGRAM]:
                 if self.twBehaviors.rowCount():
-                    response = dialog.MessageDialog(programName,
-                                                    ("Some behaviors are already configured. "
-                                                        "Do you want to append behaviors or replace them?"),
+                    response = dialog.MessageDialog(programName, ("Some behaviors are already configured. "
+                                                                  "Do you want to append behaviors or replace them?"),
                                                     ["Append", "Replace", CANCEL])
                     if response == "Replace":
                         self.twBehaviors.setRowCount(0)
@@ -431,8 +455,11 @@ def import_behaviors_from_project(self):
                                 if project[ETHOGRAM][i][field]:
                                     modif_set_list = project[ETHOGRAM][i][field].split("|")
                                     for modif_set in modif_set_list:
-                                        modif_set_dict[str(len(modif_set_dict))] = {"name": "", "type": SINGLE_SELECTION,
-                                                                                    "values": modif_set.split(",")}
+                                        modif_set_dict[str(len(modif_set_dict))] = {
+                                            "name": "",
+                                            "type": SINGLE_SELECTION,
+                                            "values": modif_set.split(",")
+                                        }
                                 project[ETHOGRAM][i][field] = dict(modif_set_dict)
 
                             item.setText(str(project[ETHOGRAM][i][field]))
@@ -449,10 +476,7 @@ def import_behaviors_from_project(self):
                 QMessageBox.warning(self, programName, "No behaviors configuration found in project")
 
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
-
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_subjects_from_project(self):
@@ -462,9 +486,8 @@ def import_subjects_from_project(self):
 
     try:
         fn = QFileDialog().getOpenFileName(self, "Import subjects from project file", "",
-                                        ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)")
-                                        )
+                                           ("Project files (*.boris *.boris.gz);;"
+                                            "All files (*)"))
         file_name = fn[0] if type(fn) is tuple else fn
 
         if file_name:
@@ -479,9 +502,8 @@ def import_subjects_from_project(self):
             if SUBJECTS in project and project[SUBJECTS]:
 
                 if self.twSubjects.rowCount():
-                    response = dialog.MessageDialog(programName,
-                                                    ("There are subjects already configured. "
-                                                    "Do you want to append subjects or replace them?"),
+                    response = dialog.MessageDialog(programName, ("There are subjects already configured. "
+                                                                  "Do you want to append subjects or replace them?"),
                                                     ['Append', 'Replace', 'Cancel'])
 
                     if response == "Replace":
@@ -506,10 +528,7 @@ def import_subjects_from_project(self):
             else:
                 QMessageBox.warning(self, programName, "No subjects configuration found in project")
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Error in function '{sys._getframe().f_code.co_name}': {error_type} {error_file_name} {error_lineno}")
-        dialog.error_message_box(sys._getframe().f_code.co_name, error_type, error_file_name, error_lineno)
-
+        dialog.error_message(sys._getframe().f_code.co_name, sys.exc_info())
 
 
 def import_indep_variables_from_project(self):
@@ -519,9 +538,8 @@ def import_indep_variables_from_project(self):
 
     try:
         fn = QFileDialog().getOpenFileName(self, "Import independent variables from project file", "",
-                                        ("Project files (*.boris *.boris.gz);;"
-                                            "All files (*)")
-                                        )
+                                           ("Project files (*.boris *.boris.gz);;"
+                                            "All files (*)"))
 
         file_name = fn[0] if type(fn) is tuple else fn
 
@@ -570,7 +588,4 @@ def import_indep_variables_from_project(self):
                 QMessageBox.warning(self, programName, "No independent variables found in project")
 
     except Exception:
-        error_type, error_file_name, error_lineno = utilities.error_info(sys.exc_info())
-        logging.critical(f"Import independent variable from project: {error_type} {error_file_name} {error_lineno}")
-
-        dialog.error_message_box("Import independent variable from project", error_type, error_file_name, error_lineno)
+        dialog.error_message("Import independent variable from project", sys.exc_info())
